# frozen_string_literal: true
require 'json_schemer'
require 'semantic'
require_relative 'version'

module Wisp
  class NotSupportedAction < StandardError
  end

  SCHEMA_PATH = File.expand_path('../wisp.json', __dir__)
  NOUNS_PATH = File.expand_path('../nouns.json', __dir__)

  CANARY_SCHEMA_PATH = File.expand_path('../canary-wisp.json', __dir__)
  CANARY_NOUNS_PATH = File.expand_path('../canary-nouns.json', __dir__)

  ACTIONS = [
    "click",
    "close_tab",
    "double_click",
    "drag_and_drop",
    "fill",
    "hover",
    "loaded",
    "navigate",
    "observe",
    "refresh",
    "scroll",
    "select",
    "send_key",
    "tester_confirmation",
    "tester_instruction",
    "type",
    "wait",
  ].freeze

  SEMANTIC_STABLE_VERSION = Semantic::Version.new(STABLE_VERSION)

  def is_canary?(raw_version)
    Semantic::Version.new(raw_version) > SEMANTIC_STABLE_VERSION
  end
  module_function :is_canary?

  def wisp_schemer(raw_version)
    if is_canary?(raw_version)
      JSONSchemer.schema(Pathname.new(Wisp::CANARY_SCHEMA_PATH))
    else
      JSONSchemer.schema(Pathname.new(Wisp::SCHEMA_PATH))
    end
  end
  module_function :wisp_schemer

  def nouns_schemer(raw_version)
    if is_canary?(raw_version)
      JSONSchemer.schema(Pathname.new(Wisp::CANARY_NOUNS_PATH))
    else
      JSONSchemer.schema(Pathname.new(Wisp::NOUNS_PATH))
    end
  end
  module_function :nouns_schemer

  def action_schemer(raw_version, action)
    raise NotSupportedAction unless ACTIONS.include?(action)

    if is_canary?(raw_version)
      JSONSchemer.schema(Pathname.new(File.expand_path("../canary/actions/#{action}.json", __dir__)))
    else
      JSONSchemer.schema(Pathname.new(File.expand_path("../stable/actions/#{action}.json", __dir__)))
    end
  end
  module_function :action_schemer
end
